bl_info = {
    'name': 'Import .mhx2 extended files',
    'author': 'BLP based on code by Thomas Larsson',
    'version': (0,30),
    "blender": (2, 79, 0),
    'location': "File > Import > MakeHuman MHX2+ (.mhx2)",
    'description': 'Import .mhx2 files with extended features',
    'warning': '',
    'wiki_url': 'http://thomasmakehuman.wordpress.com/mhx2-documentation/',
    'category': 'MakeHuman'}

if "bpy" in locals():
    # If reloading reload all the files
    print("Reloading MHX2+ importer v %d.%d" % bl_info["version"])
    import imp
    imp.reload(error)
    imp.reload(config)
    imp.reload(importer)
    imp.reload(bone_drivers)
    imp.reload(drivers)
    imp.reload(geometries)
    imp.reload(hm8)
    imp.reload(load_json)
    imp.reload(materials)
    imp.reload(proxy)
    imp.reload(shapekeys)
    imp.reload(utils)
else:
    # If loading the first time just load the needed files
    print("Loading MHX2+ importer v %d.%d" % bl_info["version"])
    from . import error
    from . import config
    from . import importer
    from . import bone_drivers
    from . import drivers
    from . import geometries
    from . import hm8
    from . import load_json
    from . import materials
    from . import proxy
    from . import shapekeys
    from . import utils

import bpy
from .error import *

# ImportHelper is a helper class, defines filename and
# invoke() function which calls the file selector.
from bpy_extras.io_utils import ImportHelper
from bpy.props import StringProperty, BoolProperty, EnumProperty, IntProperty
from bpy.types import Operator


class ImportMHX2Plus(Operator, ImportHelper):
    """This appears in the tooltip of the operator and in the generated docs"""
    bl_idname = "import_test.mhx2_plus"  # important since its how bpy.ops.import_test.some_data is constructed
    bl_label = "Import MHX2+"

    # ImportHelper mixin class uses this
    filename_ext = ".mhx2"

    filter_glob = StringProperty(
        default = "*.mhx2",
        options = {'HIDDEN'},
        maxlen = 255,  # Max internal buffer length, longer would be clamped.
    )

    # List of operator properties, the attributes will be assigned
    # to the class instance from the operator settings before calling.
    useExtensions = BoolProperty(
        name = "Use extensions",
        description = "Use the extensions to the .mhx2 loader",
        default = True,
    )
    useAutoweight = BoolProperty(
        name = "Use autoweight",
        description = "Use autoweighting for the rig",
        default = False,
    )
    useCustomWeights = BoolProperty(
        name = "Use custom weights",
        description = "Use custom weights for the rig",
        default = True,
    )
    useOffset = BoolProperty(
        name="Offset",
        description="Add offset for feet on ground",
        default=True,
    )
    useCorrectiveSmooth = BoolProperty(
        name = "Corrective Smooth",
        description = "Use corrective smooth modifier",
        default = True,
    )
    useSubsurf = BoolProperty(
        name = "Subsurf",
        description = "Use subsurf modifier",
        default = True,
    )
    subsurfViewLevels = IntProperty(
        name = "Subsurf view levels",
        description = "Levels of subsurf when viewing",
        default = 1,
    )
    subsurfRenderLevels = IntProperty(
        name = "Subsurf render levels",
        description = "Levels of subsurf when rendering",
        default = 2,
    )

    #type = EnumProperty(
    #    name="Example Enum",
    #    description="Choose between two items",
    #    items=(
    #        ('OPT_A', "First Option", "Description one"),
    #        ('OPT_B', "Second Option", "Description two"),
    #    ),
    #    default='OPT_A',
    #)

    def execute(self, context):
        from .config import Config
        cfg = Config().fromSettings(self)
        print(cfg)
        try:
            importer.importMhx2File(self.filepath, cfg, context)
        except MhxError:
            handleMhxError(context)

        AutoWeight = False #TODO
        if AutoWeight:
            scn = context.scene
            rig = scn.objects["Bar"]
            ob = scn.objects["Bar:Body"]
            ob.select = True
            bpy.ops.object.parent_set(type='ARMATURE_AUTO')

        return {'FINISHED'}


# Only needed if you want to add into a dynamic menu
def menu_func_import(self, context):
    self.layout.operator(ImportMHX2Plus.bl_idname, text="MakeHuman MHX2+ (.mhx2)")


def register():
    bpy.utils.register_class(ImportMHX2Plus)
    bpy.types.INFO_MT_file_import.append(menu_func_import)


def unregister():
    bpy.utils.unregister_class(ImportMHX2Plus)
    bpy.types.INFO_MT_file_import.remove(menu_func_import)


if __name__ == "__main__":
    register()

    # test call
    bpy.ops.import_test.mhx2_plus('INVOKE_DEFAULT')
